import os
import time
import unittest

from mock import patch

from consolemenu.screen import Screen


class TestScreen(unittest.TestCase):

    def test_clear(self):
        # clear will raise error if TERM is not set
        if os.getenv('TERM') is None:
            os.environ['TERM'] = 'xterm'
        screen = Screen()
        screen.println('Clearing screen...')
        screen.clear()

    @patch('consolemenu.screen.Screen.input', return_value='This is my Cat')
    def test_input(self, get_input_mock):
        input_string = Screen().input(prompt='This is my message')
        self.assertEqual(input_string, 'This is my Cat')

    def test_flush(self):
        screen = Screen()
        # standard printf will buffer, so output won't come until newline
        screen.println('The next line should print all at once...')
        for i in range(0, 40):
            screen.printf('.')
            time.sleep(0.5)
        screen.println()
        # now flush after each dot
        screen.println('The next line should print smoothly...')
        for i in range(0, 40):
            screen.printf('.')
            screen.flush()
            time.sleep(0.5)
        screen.println()

    def test_printf(self):
        screen = Screen()
        screen.printf('single message.')
        screen.printf('this should be on ame line as above. explicit newline: \n')
        screen.printf('this', 'is', 'a', 'list', 'message')
        screen.printf('same line as list message', 'explicit newline: \n')
        screen.printf('this is a %s message.\n' % 'printf-style')
        screen.printf('this is a {0} message.\n'.format('format-style'))

    def test_println(self):
        screen = Screen()
        screen.println('single message.')
        screen.println('this a second line.')
        screen.println('this', 'is', 'a', 'list', 'message')
        screen.println('same is a line with an explicit newline, which should cause an empty space below me.\n')
        screen.println('this is a %s message.' % 'printf-style')
        screen.println('this is a {0} message.'.format('format-style'))

    def test_screen_size(self):
        screen = Screen()
        print('screen height:', screen.screen_height)
        print('screen width:', screen.screen_width)
        self.assertEqual(40, screen.screen_height)
        self.assertEqual(80, screen.screen_width)



try:
    from unittest.mock import Mock, patch
except ImportError:
    from mock import Mock, patch

from base_test_case import BaseTestCase, ThreadedReturnGetter
from consolemenu import ConsoleMenu
from consolemenu import SelectionMenu


class TestSelectionMenu(BaseTestCase):
    def test_select(self):
        selection_menu = SelectionMenu(strings=["a", "b", "c"], title="Select a letter")
        selection_menu.start()
        selection_menu.wait_for_start(timeout=10)
        selection_menu.go_down()
        selection_menu.select()
        selection_menu.join(timeout=10)
        self.assertFalse(selection_menu.is_alive())
        self.assertEqual(selection_menu.selected_option, 1)

    def test_get_selection(self):
        menu = []
        menu_thread = ThreadedReturnGetter(SelectionMenu.get_selection, args=[["One", "Two", "Three"]],
                                           kwargs={"_menu": menu})

        menu_thread.start()
        while not menu:
            continue
        menu = menu[0]
        menu.wait_for_start(timeout=10)
        menu.go_down()
        menu.select()
        print(menu.returned_value)
        menu_thread.join(timeout=10)
        self.assertFalse(menu_thread.is_alive())
        self.assertEqual(menu_thread.return_value, 1)

    def test_current_menu(self):
        menu = []
        self.menu_thread = ThreadedReturnGetter(SelectionMenu.get_selection, args=[["One", "Two", "Three"]],
                                                kwargs={"_menu": menu})

        self.menu_thread.start()
        while not menu:
            continue
        menu = menu[0]
        menu.wait_for_start(timeout=10)
        self.assertIsInstance(ConsoleMenu.currently_active_menu, SelectionMenu)
        self.assertIs(ConsoleMenu.currently_active_menu, menu)

        selection_menu = SelectionMenu(strings=["a", "b", "c"], title="Select a letter")
        selection_menu.start()
        selection_menu.wait_for_start(timeout=10)
        self.assertIs(ConsoleMenu.currently_active_menu, selection_menu)

    def test_init(self):
        selection_menu_1 = SelectionMenu(["1", "2", "3"])
        selection_menu_2 = SelectionMenu(["4", "5"], "selection_menu_2", "test_init", show_exit_option=True)
        selection_menu_3 = SelectionMenu(strings=["6", "7", "8", "9"], title="selection_menu_3", subtitle="test_init",
                                         show_exit_option=False)
        self.assertIsNone(selection_menu_1.title)
        self.assertEqual(selection_menu_2.title, "selection_menu_2")
        self.assertEqual(selection_menu_3.title, "selection_menu_3")
        self.assertIsNone(selection_menu_1.subtitle)
        self.assertEqual(selection_menu_2.subtitle, "test_init")
        self.assertEqual(selection_menu_3.subtitle, "test_init")
        self.assertTrue(selection_menu_1.show_exit_option)
        self.assertTrue(selection_menu_2.show_exit_option)
        self.assertFalse(selection_menu_3.show_exit_option)
        self.assertEqual(selection_menu_1.items[1].text, "2")
        self.assertEqual(selection_menu_2.items[0].text, "4")
        self.assertEqual(selection_menu_3.items[3].text, "9")



from base_test_case import BaseTestCase
from consolemenu import ConsoleMenu
from consolemenu.items import SelectionItem


class TestSelectionItem(BaseTestCase):
    def setUp(self):
        super(TestSelectionItem, self).setUp()
        self.menu = ConsoleMenu("self.menu", "TestSelectionItem")

    def test_init(self):
        selection_item_1 = SelectionItem("selection_item_1", 1, self.menu)
        selection_item_2 = SelectionItem(text="selection_item_2", index=2, menu=self.menu)
        self.assertEqual(selection_item_1.text, "selection_item_1")
        self.assertEqual(selection_item_2.text, "selection_item_2")
        self.assertEqual(selection_item_1.menu, self.menu)
        self.assertEqual(selection_item_2.menu, self.menu)
        self.assertTrue(selection_item_1.should_exit)
        self.assertTrue(selection_item_2.should_exit)



from base_test_case import BaseTestCase

from consolemenu import MultiSelectMenu
from consolemenu.items import FunctionItem


class TestMultiSelectMenu(BaseTestCase):

    def setUp(self):
        super(TestMultiSelectMenu, self).setUp()

        self.menu = MultiSelectMenu("Root Menu", "This is a Multi-Select Menu")
        self.menu.append_item(FunctionItem("Action Item 1", self.detect_action, args=['one']))
        self.menu.append_item(FunctionItem("Action Item 2", self.detect_action, args=['two']))
        self.menu.append_item(FunctionItem("Action Item 3", self.detect_action, args=['three']))
        self.menu.append_item(FunctionItem("Action Item 4", self.detect_action, args=['four']))

        self.action_detector = dict()
        self.assertIsNone(self.action_detector.get('one'))
        self.assertIsNone(self.action_detector.get('two'))
        self.assertIsNone(self.action_detector.get('three'))
        self.assertIsNone(self.action_detector.get('four'))

    def detect_action(self, action):
        self.action_detector[action] = True

    def test_1and2(self):
        self.mock_screen().input.return_value = '1,2,5'
        self.menu.start()
        self.menu.wait_for_start(10)
        self.menu.join(timeout=10)

        self.assertTrue(self.action_detector.get('one'))
        self.assertTrue(self.action_detector.get('two'))
        self.assertIsNone(self.action_detector.get('three'))
        self.assertIsNone(self.action_detector.get('four'))

    def test_1and3(self):
        self.mock_screen().input.return_value = '1,3,5'
        self.menu.start()
        self.menu.wait_for_start(10)
        self.menu.join(timeout=10)

        self.assertTrue(self.action_detector.get('one'))
        self.assertIsNone(self.action_detector.get('two'))
        self.assertTrue(self.action_detector.get('three'))
        self.assertIsNone(self.action_detector.get('four'))

    def test_1to3(self):
        self.mock_screen().input.return_value = '1-3,5'
        self.menu.start()
        self.menu.wait_for_start(10)
        self.menu.join(timeout=10)

        self.assertTrue(self.action_detector.get('one'))
        self.assertTrue(self.action_detector.get('two'))
        self.assertTrue(self.action_detector.get('three'))
        self.assertIsNone(self.action_detector.get('four'))

    def test_1to4(self):
        self.mock_screen().input.return_value = '1-4,5'
        self.menu.start()
        self.menu.wait_for_start(10)
        self.menu.join(timeout=10)

        self.assertTrue(self.action_detector.get('one'))
        self.assertTrue(self.action_detector.get('two'))
        self.assertTrue(self.action_detector.get('three'))
        self.assertTrue(self.action_detector.get('four'))

    def test_2and3(self):
        self.mock_screen().input.return_value = '2,3,5'
        self.menu.start()
        self.menu.wait_for_start(10)
        self.menu.join(timeout=10)

        self.assertIsNone(self.action_detector.get('one'))
        self.assertTrue(self.action_detector.get('two'))
        self.assertTrue(self.action_detector.get('three'))
        self.assertIsNone(self.action_detector.get('four'))

    def test_3to4(self):
        self.mock_screen().input.return_value = '3-4,5'
        self.menu.start()
        self.menu.wait_for_start(10)
        self.menu.join(timeout=10)

        self.assertIsNone(self.action_detector.get('one'))
        self.assertIsNone(self.action_detector.get('two'))
        self.assertTrue(self.action_detector.get('three'))
        self.assertTrue(self.action_detector.get('four'))

    def test_4to2(self):
        self.mock_screen().input.return_value = '4-2,5'
        self.menu.start()
        self.menu.wait_for_start(10)
        self.menu.join(timeout=10)

        self.assertIsNone(self.action_detector.get('one'))
        self.assertTrue(self.action_detector.get('two'))
        self.assertTrue(self.action_detector.get('three'))
        self.assertTrue(self.action_detector.get('four'))



from base_test_case import BaseTestCase
from consolemenu import ConsoleMenu
from consolemenu.items import MenuItem


class TestMenuItem(BaseTestCase):
    def setUp(self):
        super(TestMenuItem, self).setUp()
        self.menu = ConsoleMenu("self.menu", "TestMenuItem")

    def test_init(self):
        menu_item_1 = MenuItem("menu_item_1", self.menu)
        menu_item_2 = MenuItem("menu_item_2", self.menu, True)
        menu_item_3 = MenuItem(text="menu_item_1", menu=self.menu, should_exit=False)
        self.assertEqual(menu_item_1.text, "menu_item_1")
        self.assertEqual(menu_item_2.text, "menu_item_2")
        self.assertEqual(menu_item_3.text, "menu_item_1")
        self.assertEqual(menu_item_1.menu, self.menu)
        self.assertEqual(menu_item_2.menu, self.menu)
        self.assertEqual(menu_item_3.menu, self.menu)
        self.assertFalse(menu_item_1.should_exit)
        self.assertTrue(menu_item_2.should_exit)
        self.assertFalse(menu_item_3.should_exit)

    def test_show(self):
        menu_item = MenuItem("menu_item", self.menu)
        self.assertEqual(menu_item.show(0), " 1 - menu_item")



from __future__ import print_function

from base_test_case import BaseTestCase
from consolemenu import ConsoleMenu, MenuFormatBuilder
from consolemenu.format.menu_style import (
    MenuStyle
)
from consolemenu.format.menu_borders import LightBorderStyle
from consolemenu.format.menu_padding import MenuPadding
from consolemenu.format.menu_margins import MenuMargins
from consolemenu.items import MenuItem
from consolemenu.menu_component import MenuHeader, MenuTextSection, MenuItemsSection, MenuFooter, MenuPrompt
from consolemenu.screen import Screen


def print_screen_edge(width=80):
    msg = '{title:{fill}^{width}}'.format(title='simulate screen edges', fill='-', width=(width - 2))
    print('{edge}{msg}{edge}'.format(edge="|", msg=msg))


class TestMenuFormatBuilder(BaseTestCase):

    def test_empty(self):
        Screen().printf(MenuFormatBuilder().format())

    def test_defaults(self):
        print_screen_edge()
        format = MenuFormatBuilder()
        items = [MenuItem("This is Item 1"),
                 MenuItem("This is Item 2"),
                 MenuItem("This is Item 3")]
        Screen().printf(format.format(title="This is My Title",
                                      subtitle="This is My Little Subtitle",
                                      items=items))

    def test_format_with_prologue_no_border(self):
        format = MenuFormatBuilder()
        items = [MenuItem("This is Item 1"),
                 MenuItem("This is Item 2"),
                 MenuItem("This is Item 3")]
        prologue_text = 'This a very long prologue, which can be used to explain how to use this menu, \
for people that might not understand it.'
        Screen().printf(format.format(title="This is My Title", subtitle="This is My Subtitle",
                                      items=items, prologue_text=prologue_text))

    def test_format_with_header_bottom_border_and_prologue_top_border(self):
        format = MenuFormatBuilder().show_header_bottom_border(True).show_prologue_top_border(True)
        items = [MenuItem("This is Item 1"),
                 MenuItem("This is Item 2"),
                 MenuItem("This is Item 3")]
        prologue_text = 'This a very long prologue, which can be used to explain how to use this menu, \
for people that might not understand it.'
        Screen().printf(format.format(title="This is My Title", subtitle="This is My Subtitle",
                                      items=items, prologue_text=prologue_text))

    def test_format_with_prologue_with_top_border(self):
        format = MenuFormatBuilder().show_prologue_top_border(True)
        items = [MenuItem("This is Item 1"),
                 MenuItem("This is Item 2"),
                 MenuItem("This is Item 3")]
        prologue_text = "This is my prologue. Follow these instructions."
        Screen().printf(format.format(title="This is My Title", subtitle="This is My Subtitle",
                                      items=items, prologue_text=prologue_text))

    def test_format_with_prologue_with_top_and_bottom_borders(self):
        format = MenuFormatBuilder().show_prologue_top_border(True).show_prologue_bottom_border(True)
        items = [MenuItem("This is Item 1"),
                 MenuItem("This is Item 2"),
                 MenuItem("This is Item 3")]
        prologue_text = "This is my prologue. Follow these instructions."
        Screen().printf(format.format(title="This is My Title", subtitle="This is My Subtitle",
                                      items=items, prologue_text=prologue_text))

    def test_format_with_long_prologue_with_both_borders(self):
        format = MenuFormatBuilder().show_prologue_top_border(True).show_prologue_bottom_border(True)
        items = [MenuItem("This is Item 1"),
                 MenuItem("This is Item 2"),
                 MenuItem("This is Item 3")]
        prologue_text = 'This a very long prologue, which can be used to explain how to use this menu, \
for people that might not understand it. But if they read this description it can help them. Also, I have both \
my top and bottom borders enabled, so you should see them.'
        Screen().printf(format.format(title="This is My Title", subtitle="This is My Subtitle",
                                      items=items, prologue_text=prologue_text))

    def test_format_with_item_borders(self):
        format = MenuFormatBuilder()
        item1 = MenuItem("This is Item 1")
        item2 = MenuItem("This is Item 2")
        item3 = MenuItem("This is Item 3")
        format.show_item_top_border(item2, True)
        format.show_item_bottom_border(item2, True)
        print("This should show both top and bottom borders on item 2...")
        Screen().printf(format.format(title="This is My Title", subtitle="This is My Subtitle",
                                      items=[item1, item2, item3]))
        # Now turn off the borders
        format.show_item_top_border(item2, False)
        format.show_item_bottom_border(item2, False)
        print("This should NOT show any borders on item 2...")
        Screen().printf(format.format(title="This is My Title", subtitle="This is My Subtitle",
                                      items=[item1, item2, item3]))



import sys
from threading import Thread
from consolemenu.screen import Screen

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

try:
    from unittest.mock import Mock, patch
except ImportError:
    from mock import Mock, patch


class ThreadedReturnGetter(Thread):

    def __init__(self, function, args=None, kwargs=None):
        if args is None:
            args = []
        if kwargs is None:
            kwargs = {}
        self.return_value = None
        self.function = function
        try:
            super(ThreadedReturnGetter, self).__init__(target=self.get_return_value, args=args, kwargs=kwargs,
                                                       daemon=True)
        except TypeError:
            super(ThreadedReturnGetter, self).__init__(target=self.get_return_value, args=args, kwargs=kwargs)
            self.daemon = True

    def get_return_value(self, *args, **kwargs):
        self.return_value = self.function(*args, **kwargs)


class BaseTestCase(unittest.TestCase):

    def setUp(self):
        self.mock_screen = Mock(spec=Screen())
        self.mock_screen.input.return_value = 4

        self.patcher = patch(target='consolemenu.console_menu.Screen', new=self.mock_screen)
        self.patcher.start()
        self.addCleanup(self.patcher.stop)



import os.path
import platform

from consolemenu import ConsoleMenu

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

from consolemenu.items import CommandItem
from test_external_item import TestExternalItem


class TestCommandItem(TestExternalItem):
    def setUp(self):
        super(TestCommandItem, self).setUp()
        self.menu = ConsoleMenu("self.menu", "TestCommandItem")

    def test_init(self):
        command_item_1 = CommandItem("command_item_1", "exit")
        command_item_2 = CommandItem("command_item_2", "ls", ["-l", "-a", "~"], self.menu, True)
        command_item_3 = CommandItem(text="command_item_3", command="rm", menu=self.menu,
                                     arguments=["-r", "-f", "./test"], should_exit=False)

        self.assertEqual(command_item_1.text, "command_item_1")
        self.assertEqual(command_item_2.text, "command_item_2")
        self.assertEqual(command_item_3.text, "command_item_3")
        self.assertEqual(command_item_1.menu, None)
        self.assertEqual(command_item_2.menu, self.menu)
        self.assertEqual(command_item_3.menu, self.menu)
        self.assertFalse(command_item_1.should_exit)
        self.assertTrue(command_item_2.should_exit)
        self.assertFalse(command_item_3.should_exit)
        self.assertEqual(command_item_1.command, "exit")
        self.assertEqual(command_item_2.command, "ls")
        self.assertEqual(command_item_3.command, "rm")
        self.assertEqual(command_item_1.arguments, [])
        self.assertEqual(command_item_2.arguments, ["-l", "-a", "~"])
        self.assertEqual(command_item_3.arguments, ["-r", "-f", "./test"])

    def test_return(self):
        if platform.system().lower() == "windows":
            return_command_item = CommandItem("return_command_item", "exit 1")
        else:
            return_command_item = CommandItem("return_command_item", "exit 1")

        return_command_item.action()

        self.assertEqual(return_command_item.get_return(), 1)

    def test_run(self):
        create_item = CommandItem("create_item", 'echo hello>test.txt')
        if platform.system().lower() == "windows":
            delete_item = CommandItem("delete_item", "del test.txt")
            expected_contents = "hello \n"
        else:
            delete_item = CommandItem("delete_item", "rm test.txt")
            expected_contents = "hello\n"
        create_item.action()
        self.assertEqual(create_item.get_return(), 0)
        self.assertTrue(os.path.isfile("test.txt"))

        with open("test.txt", 'r') as text:
            self.assertEqual(text.read(), expected_contents)

        delete_item.action()
        self.assertEqual(delete_item.get_return(), 0)
        self.assertFalse(os.path.isfile("test.txt"))

    @patch('consolemenu.items.command_item.subprocess')
    def test_call(self, mock_class):
        command_item = CommandItem("command_item", "ls", ["-l", "-a", "~"])
        command_item.action()
        mock_class.run.assert_called_with("ls -l -a ~", shell=True)



from __future__ import print_function

import unittest

from mock import Mock, patch

from consolemenu import Screen, PromptUtils
from consolemenu import UserQuit
from consolemenu.validators.base import InvalidValidator
from consolemenu.validators.regex import RegexValidator
from consolemenu.validators.url import UrlValidator


class PromptUtilsTest(unittest.TestCase):

    def setUp(self):
        self.mock_screen = Mock(spec=Screen())
        self.mock_screen.input.return_value = 4

    def test_prompt_for_numbered_choice_list(self):
        prompt_utils = PromptUtils(self.mock_screen)
        choices = [
            'option_1', 'option_2', 'option_3', 'option_4', 'option_5', 'option_6',
            'option_7', 'option_8', 'option_9', 'option_10', 'option_11', 'option_12'
        ]
        # Test for input of 1
        self.mock_screen.input.return_value = 1
        selected = prompt_utils.prompt_for_numbered_choice(choices)
        self.assertEqual(0, selected)
        self.assertEqual('option_1', choices[selected])

        # Test for input of 3
        self.mock_screen.input.return_value = 3
        selected = prompt_utils.prompt_for_numbered_choice(choices)
        self.assertEqual(2, selected)
        self.assertEqual('option_3', choices[selected])

        # Test for input of 7
        self.mock_screen.input.return_value = 7
        selected = prompt_utils.prompt_for_numbered_choice(choices)
        self.assertEqual(6, selected)
        self.assertEqual('option_7', choices[selected])

    def test_enter_to_continue(self):
        prompt_utils = PromptUtils(self.mock_screen)
        self.mock_screen.input.return_value = "\n"

        print('First, using the default message...')
        prompt_utils.enter_to_continue()

        print('Now using the specified message...')
        prompt_utils.enter_to_continue('My custom prompt     ')

    def test_prompt_for_bilateral_choice(self):
        prompt_utils = PromptUtils(self.mock_screen)

        self.mock_screen.input.return_value = "yes"
        answer = prompt_utils.prompt_for_bilateral_choice('Please enter a value', 'yes', 'no')
        self.assertEqual('yes', answer)

        self.mock_screen.input.return_value = "Yes"
        answer = prompt_utils.prompt_for_bilateral_choice('Please enter a value', 'yes', 'no')
        self.assertEqual('yes', answer)

        self.mock_screen.input.return_value = "yEs"
        answer = prompt_utils.prompt_for_bilateral_choice('Please enter a value', 'yes', 'no')
        self.assertEqual('yes', answer)

        self.mock_screen.input.return_value = "YES"
        answer = prompt_utils.prompt_for_bilateral_choice('Please enter a value', 'yes', 'no')
        self.assertEqual('yes', answer)

        self.mock_screen.input.return_value = "NO"
        answer = prompt_utils.prompt_for_bilateral_choice('Please enter a value', 'yes', 'no')
        self.assertEqual('no', answer)

        self.mock_screen.input.return_value = "no"
        answer = prompt_utils.prompt_for_bilateral_choice('Please enter a value', 'yes', 'no')
        self.assertEqual('no', answer)

        self.mock_screen.input.return_value = "Y"
        answer = prompt_utils.prompt_for_bilateral_choice('Another prompt, with letters', 'Y', 'N')
        self.assertEqual('Y', answer)

        self.mock_screen.input.return_value = "y"
        answer = prompt_utils.prompt_for_bilateral_choice('Another prompt, with letters', 'Y', 'N')
        self.assertEqual('Y', answer)

        self.mock_screen.input.return_value = "N"
        answer = prompt_utils.prompt_for_bilateral_choice('Another prompt, with letters', 'Y', 'N')
        self.assertEqual('N', answer)

        self.mock_screen.input.return_value = "n"
        answer = prompt_utils.prompt_for_bilateral_choice('Another prompt, with letters', 'Y', 'N')
        self.assertEqual('N', answer)

        self.mock_screen.input.return_value = "e"
        answer = prompt_utils.prompt_for_bilateral_choice('Enable/Disable this setting', 'E', 'D')
        self.assertEqual('E', answer)

        self.mock_screen.input.return_value = "d"
        answer = prompt_utils.prompt_for_bilateral_choice('Enable/Disable this setting', 'E', 'D')
        self.assertEqual('D', answer)

    def test_prompt_for_trilateral_choice(self):
        prompt_utils = PromptUtils(self.mock_screen)

        self.mock_screen.input.return_value = "YES"
        answer = prompt_utils.prompt_for_trilateral_choice('Please enter a value', 'yes', 'no', 'maybe')
        self.assertEqual('yes', answer)

        self.mock_screen.input.return_value = "NO"
        answer = prompt_utils.prompt_for_trilateral_choice('Please enter a value', 'yes', 'no', 'maybe')
        self.assertEqual('no', answer)

        self.mock_screen.input.return_value = "maYbe"
        answer = prompt_utils.prompt_for_trilateral_choice('Please enter a value', 'yes', 'no', 'maybe')
        self.assertEqual('maybe', answer)

        self.mock_screen.input.return_value = "y"
        answer = prompt_utils.prompt_for_trilateral_choice('Another prompt, with letters', 'Y', 'N', 'M')
        self.assertEqual('Y', answer)

        self.mock_screen.input.return_value = "e"
        answer = prompt_utils.prompt_for_trilateral_choice('Enable/Disable/Configure this setting', 'E', 'D', 'C')
        self.assertEqual('E', answer)

        self.mock_screen.input.return_value = "d"
        answer = prompt_utils.prompt_for_trilateral_choice('Enable/Disable/Configure this setting', 'E', 'D', 'C')
        self.assertEqual('D', answer)

        self.mock_screen.input.return_value = "c"
        answer = prompt_utils.prompt_for_trilateral_choice('Enable/Disable/Configure this setting', 'E', 'D', 'C')
        self.assertEqual('C', answer)

    def test_prompt_for_yes_or_no(self):
        prompt_utils = PromptUtils(self.mock_screen)

        self.mock_screen.input.return_value = "y"
        answer = prompt_utils.prompt_for_yes_or_no('Do you want to continue?')
        self.assertTrue(answer)

        self.mock_screen.input.return_value = "n"
        answer = prompt_utils.prompt_for_yes_or_no('Do you want to continue?')
        self.assertFalse(answer)

    def test_input(self):
        prompt_utils = PromptUtils(self.mock_screen)

        # Test the default answer when user just hits enter
        self.mock_screen.input.return_value = "\n"
        res = prompt_utils.input('This is my message with a default', 'default answer')
        self.assertEqual('default answer', res.input_string)

        self.mock_screen.input.return_value = "my_fake_input"
        res = prompt_utils.input('This is my message with no default')
        self.assertEqual('my_fake_input', res.input_string)

    @patch('consolemenu.screen.Screen.input', return_value='This is my Cat')
    def test_input_validation_regex_true(self, get_input_mock):
        prompt_utils = PromptUtils(Screen())
        input_result = prompt_utils.input(prompt='This is my message', validators=RegexValidator(pattern='.*Cat.*'))
        self.assertTrue(input_result.validation_result)
        self.assertEqual(input_result.input_string, 'This is my Cat')

    @patch('consolemenu.screen.Screen.input', return_value='This is my Cat')
    def test_input_validation_regex_false(self, get_input_mock):
        prompt_utils = PromptUtils(Screen())
        input_result = prompt_utils.input(prompt='This is my message', validators=RegexValidator(pattern='Cat'))
        self.assertFalse(input_result.validation_result)
        self.assertEqual(input_result.input_string, 'This is my Cat')

    @patch('consolemenu.screen.Screen.input', return_value='https://www.google.com')
    def test_input_validation_regex_and_url_one_false(self, get_input_mock):
        prompt_utils = PromptUtils(Screen())
        input_result = prompt_utils.input(prompt='This is my message',
                                          validators=[RegexValidator(pattern='notpresent'), UrlValidator()])
        self.assertFalse(input_result.validation_result)
        self.assertEqual(input_result.input_string, 'https://www.google.com')

    @patch('consolemenu.screen.Screen.input', return_value='https://www.google.com')
    def test_input_validation_regex_and_url_all_false(self, get_input_mock):
        prompt_utils = PromptUtils(Screen())
        input_result = prompt_utils.input(prompt='This is my message',
                                          validators=[RegexValidator(pattern='notpresent'), UrlValidator()])
        self.assertFalse(input_result.validation_result)
        self.assertEqual(input_result.input_string, 'https://www.google.com')

    @patch('consolemenu.screen.Screen.input', return_value='https://asdasd')
    def test_input_validation_emtpy_list(self, get_input_mock):
        prompt_utils = PromptUtils(Screen())
        input_result = prompt_utils.input(prompt='This is my message', validators=[])
        self.assertTrue(input_result.validation_result)
        self.assertEqual(input_result.input_string, 'https://asdasd')

    @patch('consolemenu.screen.Screen.input', return_value='https://asdasd')
    def test_input_validation_invalid_validation(self, get_input_mock):
        prompt_utils = PromptUtils(Screen())
        with self.assertRaises(InvalidValidator):
            prompt_utils.input(prompt='This is my message', validators=[None])

    @patch('consolemenu.screen.Screen.input', return_value='https://asdasd')
    def test_input_validators_None(self, get_input_mock):
        prompt_utils = PromptUtils(Screen())
        input_result = prompt_utils.input(prompt='This is my message', validators=None)
        self.assertTrue(input_result.validation_result)
        self.assertEqual(input_result.input_string, 'https://asdasd')

    @patch('consolemenu.screen.Screen.input', return_value='')
    def test_input_validators_with_default(self, get_input_mock):
        prompt_utils = PromptUtils(Screen())
        input_result = prompt_utils.input(prompt='This is my message', validators=UrlValidator(),
                                          default='https://www.google.com')
        self.assertTrue(input_result.validation_result)
        self.assertEqual(input_result.input_string, 'https://www.google.com')

    @patch('consolemenu.screen.Screen.input', return_value='q')
    def test_input_quit_enabled_default(self, get_input_mock):
        prompt_utils = PromptUtils(Screen())
        with self.assertRaises(UserQuit):
            prompt_utils.input(prompt='This is my message', validators=UrlValidator(),
                               default='https://www.google.com', enable_quit=True)

    @patch('consolemenu.screen.Screen.input', return_value='exit')
    def test_input_quit_enabled_none_default(self, get_input_mock):
        prompt_utils = PromptUtils(Screen())
        with self.assertRaises(UserQuit):
            prompt_utils.input(prompt='This is my message', validators=UrlValidator(),
                               default='https://www.google.com', enable_quit=True, quit_string='exit')

    @patch('consolemenu.screen.Screen.input', return_value='https://www.google.com')
    def test_input_quit_enabled_default_not_quit(self, get_input_mock):
        prompt_utils = PromptUtils(Screen())
        input_result = prompt_utils.input(prompt='This is my message', validators=UrlValidator(),
                                          default='https://www.google.com', enable_quit=True)
        self.assertTrue(input_result.validation_result)
        self.assertEqual(input_result.input_string, 'https://www.google.com')

    @patch('getpass.getpass', return_value='p@ssw0rd')
    def test_input_password(self, mock_getpass):
        prompt_utils = PromptUtils(self.mock_screen)
        ans = prompt_utils.input_password('Enter your password to be masked')
        print('you answered:', ans)
        self.assertEqual('p@ssw0rd', ans)



from unittest import TestCase

from consolemenu.validators.base import BaseValidator
from consolemenu.validators.url import UrlValidator


class UrlValidatorTestCase(TestCase):
    def test_is_child_from_base(self):
        self.assertTrue(issubclass(UrlValidator, BaseValidator))

    def test_validate_valid_https(self):
        self.assertTrue(UrlValidator().validate(input_string='https://www.google.es/'))

    def test_validate_valid_http(self):
        self.assertTrue(UrlValidator().validate(input_string='http://www.google.es/'))

    def test_validate_valid_no_ending_slash(self):
        self.assertTrue(UrlValidator().validate(input_string='https://www.google.es'))

    def test_validate_invalid(self):
        self.assertFalse(UrlValidator().validate(input_string='google'))



from base_test_case import BaseTestCase
from consolemenu import ConsoleMenu
from consolemenu.items import ExitItem


class TestExitItem(BaseTestCase):
    def test_init(self):
        menu = ConsoleMenu("menu", "test_init")
        exit_item_1 = ExitItem("exit_item_1")
        exit_item_2 = ExitItem("exit_item_2", menu)
        exit_item_3 = ExitItem(text="exit_item_3", menu=menu)
        self.assertEqual(exit_item_1.text, "exit_item_1")
        self.assertEqual(exit_item_2.text, "exit_item_2")
        self.assertEqual(exit_item_3.text, "exit_item_3")
        self.assertEqual(exit_item_1.menu, None)
        self.assertEqual(exit_item_2.menu, menu)
        self.assertEqual(exit_item_3.menu, menu)
        self.assertTrue(exit_item_1.should_exit)
        self.assertTrue(exit_item_2.should_exit)
        self.assertTrue(exit_item_3.should_exit)



from __future__ import print_function

from unittest import TestCase

from consolemenu.format.menu_borders import *


class TestAsciiBorderStyle(TestCase):

    def test(self):
        border = AsciiBorderStyle()
        # print(border.bottom_left_corner)
        # print(border.bottom_right_corner)
        # print(border.top_left_corner)
        # print(border.top_right_corner)
        # print(border.outer_horizontal)
        # print(border.outer_horizontal_inner_down)
        # print(border.outer_horizontal_inner_up)
        # print(border.outer_vertical)
        # print(border.outer_vertical_inner_left)
        # print(border.outer_vertical_inner_right)

        self.assertEqual('+', border.bottom_left_corner)
        self.assertEqual('+', border.bottom_right_corner)
        self.assertEqual('+', border.top_left_corner)
        self.assertEqual('+', border.top_right_corner)
        self.assertEqual('-', border.outer_horizontal)
        self.assertEqual('+', border.outer_horizontal_inner_down)
        self.assertEqual('+', border.outer_horizontal_inner_up)
        self.assertEqual('|', border.outer_vertical)
        self.assertEqual('|', border.outer_vertical_inner_left)
        self.assertEqual('|', border.outer_vertical_inner_right)


class TestLightBorderStyle(TestCase):

    def test(self):
        border = LightBorderStyle()
        # print border.bottom_left_corner
        # print border.bottom_right_corner
        # print border.top_left_corner
        # print border.top_right_corner
        # print border.outer_horizontal
        # print border.outer_horizontal_inner_down
        # print border.outer_horizontal_inner_up
        # print border.outer_vertical
        # print border.outer_vertical_inner_left
        # print border.outer_vertical_inner_right

        self.assertEqual(u'\u2514', border.bottom_left_corner)
        self.assertEqual(u'\u2518', border.bottom_right_corner)
        self.assertEqual(u'\u250C', border.top_left_corner)
        self.assertEqual(u'\u2510', border.top_right_corner)
        self.assertEqual(u'\u2500', border.outer_horizontal)
        self.assertEqual(u'\u252C', border.outer_horizontal_inner_down)
        self.assertEqual(u'\u2534', border.outer_horizontal_inner_up)
        self.assertEqual(u'\u2502', border.outer_vertical)
        self.assertEqual(u'\u2524', border.outer_vertical_inner_left)
        self.assertEqual(u'\u251C', border.outer_vertical_inner_right)


class TestHeavyBorderStyle(TestCase):

    def test(self):
        border = HeavyBorderStyle()
        # print(border.bottom_left_corner)
        # print(border.bottom_right_corner)
        # print(border.top_left_corner)
        # print(border.top_right_corner)
        # print(border.outer_horizontal)
        # print(border.outer_horizontal_inner_down)
        # print(border.outer_horizontal_inner_up)
        # print(border.outer_vertical)
        # print(border.outer_vertical_inner_left)
        # print(border.outer_vertical_inner_right)

        self.assertEqual(u'\u2517', border.bottom_left_corner)
        self.assertEqual(u'\u251B', border.bottom_right_corner)
        self.assertEqual(u'\u2501', border.inner_horizontal)
        self.assertEqual(u'\u2503', border.inner_vertical)
        self.assertEqual(u'\u254B', border.intersection)
        self.assertEqual(u'\u2501', border.outer_horizontal)
        self.assertEqual(u'\u2533', border.outer_horizontal_inner_down)
        self.assertEqual(u'\u253B', border.outer_horizontal_inner_up)
        self.assertEqual(u'\u2503', border.outer_vertical)
        self.assertEqual(u'\u252B', border.outer_vertical_inner_left)
        self.assertEqual(u'\u2523', border.outer_vertical_inner_right)
        self.assertEqual(u'\u250F', border.top_left_corner)
        self.assertEqual(u'\u2513', border.top_right_corner)


class TestHeavyOuterLightInnerBorderStyle(TestCase):

    def test(self):
        border = HeavyOuterLightInnerBorderStyle()
        # print(border.bottom_left_corner)
        # print(border.bottom_right_corner)
        # print(border.top_left_corner)
        # print(border.top_right_corner)
        # print(border.outer_horizontal)
        # print(border.outer_horizontal_inner_down)
        # print(border.outer_horizontal_inner_up)
        # print(border.outer_vertical)
        # print(border.outer_vertical_inner_left)
        # print(border.outer_vertical_inner_right)

        self.assertEqual(u'\u2517', border.bottom_left_corner)
        self.assertEqual(u'\u251B', border.bottom_right_corner)
        self.assertEqual(u'\u2500', border.inner_horizontal)
        self.assertEqual(u'\u2502', border.inner_vertical)
        self.assertEqual(u'\u253C', border.intersection)
        self.assertEqual(u'\u2501', border.outer_horizontal)
        self.assertEqual(u'\u252F', border.outer_horizontal_inner_down)
        self.assertEqual(u'\u2537', border.outer_horizontal_inner_up)
        self.assertEqual(u'\u2503', border.outer_vertical)
        self.assertEqual(u'\u2528', border.outer_vertical_inner_left)
        self.assertEqual(u'\u2520', border.outer_vertical_inner_right)
        self.assertEqual(u'\u250F', border.top_left_corner)
        self.assertEqual(u'\u2513', border.top_right_corner)


class TestDoubleLineBorderStyle(TestCase):

    def test(self):
        border = DoubleLineBorderStyle()
        # print(border.bottom_left_corner)
        # print(border.bottom_right_corner)
        # print(border.top_left_corner)
        # print(border.top_right_corner)
        # print(border.outer_horizontal)
        # print(border.outer_horizontal_inner_down)
        # print(border.outer_horizontal_inner_up)
        # print(border.outer_vertical)
        # print(border.outer_vertical_inner_left)
        # print(border.outer_vertical_inner_right)

        self.assertEqual(u'\u255A', border.bottom_left_corner)
        self.assertEqual(u'\u255D', border.bottom_right_corner)
        self.assertEqual(u'\u2550', border.inner_horizontal)
        self.assertEqual(u'\u2551', border.inner_vertical)
        self.assertEqual(u'\u256C', border.intersection)
        self.assertEqual(u'\u2550', border.outer_horizontal)
        self.assertEqual(u'\u2566', border.outer_horizontal_inner_down)
        self.assertEqual(u'\u2569', border.outer_horizontal_inner_up)
        self.assertEqual(u'\u2551', border.outer_vertical)
        self.assertEqual(u'\u2563', border.outer_vertical_inner_left)
        self.assertEqual(u'\u2560', border.outer_vertical_inner_right)
        self.assertEqual(u'\u2554', border.top_left_corner)
        self.assertEqual(u'\u2557', border.top_right_corner)


class TestDoubleLineOuterLightInnerBorderStyle(TestCase):

    def test(self):
        border = DoubleLineOuterLightInnerBorderStyle()
        # print(border.bottom_left_corner)
        # print(border.bottom_right_corner)
        # print(border.top_left_corner)
        # print(border.top_right_corner)
        # print(border.outer_horizontal)
        # print(border.outer_horizontal_inner_down)
        # print(border.outer_horizontal_inner_up)
        # print(border.outer_vertical)
        # print(border.outer_vertical_inner_left)
        # print(border.outer_vertical_inner_right)

        self.assertEqual(u'\u255A', border.bottom_left_corner)
        self.assertEqual(u'\u255D', border.bottom_right_corner)
        self.assertEqual(u'\u2500', border.inner_horizontal)
        self.assertEqual(u'\u2502', border.inner_vertical)
        self.assertEqual(u'\u253C', border.intersection)
        self.assertEqual(u'\u2550', border.outer_horizontal)
        self.assertEqual(u'\u2564', border.outer_horizontal_inner_down)
        self.assertEqual(u'\u2567', border.outer_horizontal_inner_up)
        self.assertEqual(u'\u2551', border.outer_vertical)
        self.assertEqual(u'\u2562', border.outer_vertical_inner_left)
        self.assertEqual(u'\u255F', border.outer_vertical_inner_right)
        self.assertEqual(u'\u2554', border.top_left_corner)
        self.assertEqual(u'\u2557', border.top_right_corner)


class TestMenuBorderStyleType(TestCase):

    def test(self):
        self.assertEqual(0, MenuBorderStyleType.ASCII_BORDER)
        self.assertEqual(1, MenuBorderStyleType.LIGHT_BORDER)
        self.assertEqual(2, MenuBorderStyleType.HEAVY_BORDER)
        self.assertEqual(3, MenuBorderStyleType.DOUBLE_LINE_BORDER)
        self.assertEqual(4, MenuBorderStyleType.HEAVY_OUTER_LIGHT_INNER_BORDER)
        self.assertEqual(5, MenuBorderStyleType.DOUBLE_LINE_OUTER_LIGHT_INNER_BORDER)


class TestMenuBorderFactory(TestCase):

    @staticmethod
    def _is_win_python35_or_earlier():
        return sys.platform.startswith("win") and sys.version_info.major < 3 or \
                (sys.version_info.major == 3 and sys.version_info.minor < 6)

    def test_create_ascii_border(self):
        style = MenuBorderStyleFactory().create_ascii_border()
        self.assertTrue(isinstance(style, AsciiBorderStyle))

    def test_create_light_border(self):
        style = MenuBorderStyleFactory().create_light_border()
        self.assertTrue(isinstance(style, LightBorderStyle))

    def test_create_heavy_border(self):
        style = MenuBorderStyleFactory().create_heavy_border()
        # On Windows/Python prior to 3.5, this will create a double-line border.
        if self._is_win_python35_or_earlier():
            self.assertTrue(isinstance(style, DoubleLineBorderStyle))
        else:
            self.assertTrue(isinstance(style, HeavyBorderStyle))

    def test_create_heavy_outer_light_inner_border(self):
        style = MenuBorderStyleFactory().create_heavy_outer_light_inner_border()
        # On Windows/Python prior to 3.5, this will create a double-line outer light inner.
        if self._is_win_python35_or_earlier():
            self.assertTrue(isinstance(style, DoubleLineOuterLightInnerBorderStyle))
        else:
            self.assertTrue(isinstance(style, HeavyOuterLightInnerBorderStyle))

    def test_create_doubleline_border(self):
        style = MenuBorderStyleFactory().create_doubleline_border()
        self.assertTrue(isinstance(style, DoubleLineBorderStyle))

    def test_create_doubleline_outer_light_border(self):
        style = MenuBorderStyleFactory().create_doubleline_outer_light_inner_border()
        self.assertTrue(isinstance(style, DoubleLineOuterLightInnerBorderStyle))

    def test_create_border(self):
        style = MenuBorderStyleFactory().create_border(MenuBorderStyleType.ASCII_BORDER)
        self.assertTrue(isinstance(style, AsciiBorderStyle))

        style = MenuBorderStyleFactory().create_border(MenuBorderStyleType.LIGHT_BORDER)
        self.assertTrue(isinstance(style, LightBorderStyle))

        style = MenuBorderStyleFactory().create_border(MenuBorderStyleType.HEAVY_BORDER)
        # On Windows/Python prior to 3.5, this will create a double-line outer light inner.
        if self._is_win_python35_or_earlier():
            self.assertTrue(isinstance(style, DoubleLineBorderStyle))
        else:
            self.assertTrue(isinstance(style, HeavyBorderStyle))

        style = MenuBorderStyleFactory().create_border(MenuBorderStyleType.HEAVY_OUTER_LIGHT_INNER_BORDER)
        # On Windows/Python prior to 3.5, this will create a double-line outer light inner.
        if self._is_win_python35_or_earlier():
            self.assertTrue(isinstance(style, DoubleLineOuterLightInnerBorderStyle))
        else:
            self.assertTrue(isinstance(style, HeavyOuterLightInnerBorderStyle))

        style = MenuBorderStyleFactory().create_border(MenuBorderStyleType.DOUBLE_LINE_BORDER)
        self.assertTrue(isinstance(style, DoubleLineBorderStyle))

        style = MenuBorderStyleFactory().create_border(MenuBorderStyleType.DOUBLE_LINE_OUTER_LIGHT_INNER_BORDER)
        self.assertTrue(isinstance(style, DoubleLineOuterLightInnerBorderStyle))



from unittest import TestCase

from consolemenu.validators.base import BaseValidator
from consolemenu.validators.regex import RegexValidator


class RegexValidatorTestCase(TestCase):

    def test_is_child_from_base(self):
        self.assertTrue(issubclass(RegexValidator, BaseValidator))

    def test_validate_match(self):
        regex_validator = RegexValidator(pattern=r'Cats')
        self.assertTrue(regex_validator.validate(input_string="Cats are smarter than dogs"))

    def test_validate_do_not_match(self):
        regex_validator = RegexValidator(pattern=r'Rats')
        self.assertFalse(regex_validator.validate(input_string="Cats are smarter than dogs"))

    def test_validate_invalid_pattern(self):
        regex_validator = RegexValidator(pattern=None)
        self.assertFalse(regex_validator.validate(input_string="Cats are smarter than dogs", ))



try:
    from unittest.mock import MagicMock
except ImportError:
    from mock import MagicMock
from base_test_case import BaseTestCase
from consolemenu import ConsoleMenu
from consolemenu.items import ExternalItem


class TestExternalItem(BaseTestCase):
    def setUp(self):
        super(TestExternalItem, self).setUp()

        self.mock_set_up = MagicMock()
        self.mock_action = MagicMock(return_value=0)
        self.mock_clean_up = MagicMock()
        ExternalItem.set_up = self.mock_set_up
        ExternalItem.clean_up = self.mock_clean_up
        ExternalItem.action = self.mock_action



from base_test_case import BaseTestCase
from consolemenu import ConsoleMenu
from consolemenu.items import SubmenuItem


class TestSubmenuItem(BaseTestCase):
    def test_init(self):
        root_menu = ConsoleMenu("root_menu", "test_init")
        submenu1 = ConsoleMenu("submenu1", "test_init")
        submenu2 = ConsoleMenu("submenu2", "test_init")
        submenu3 = ConsoleMenu("submenu2", "test_init")
        menu_item_1 = SubmenuItem("menu_item_1", submenu1)
        menu_item_2 = SubmenuItem("menu_item_2", submenu2, root_menu, True)
        menu_item_3 = SubmenuItem(text="menu_item_3", submenu=submenu3, menu=root_menu, should_exit=False)
        self.assertEqual(menu_item_1.text, "menu_item_1")
        self.assertEqual(menu_item_2.text, "menu_item_2")
        self.assertEqual(menu_item_3.text, "menu_item_3")
        self.assertIsNone(menu_item_1.menu)
        self.assertEqual(menu_item_2.menu, root_menu)
        self.assertEqual(menu_item_3.menu, root_menu)
        self.assertFalse(menu_item_1.should_exit)
        self.assertTrue(menu_item_2.should_exit)
        self.assertFalse(menu_item_3.should_exit)
        self.assertEqual(menu_item_1.submenu, submenu1)
        self.assertEqual(menu_item_2.submenu, submenu2)
        self.assertEqual(menu_item_3.submenu, submenu3)
        self.assertIsNone(menu_item_1.submenu.parent)
        self.assertEqual(menu_item_2.submenu.parent, root_menu)
        self.assertEqual(menu_item_3.submenu.parent, root_menu)

    def test_action(self):
        root_menu = ConsoleMenu("root_menu", "test_action")
        submenu1 = ConsoleMenu("submenu1", "test_action")
        submenu2 = ConsoleMenu("submenu2", "test_action")
        submenu_item_1 = SubmenuItem("submenu_item_1", submenu1, menu=root_menu)
        submenu_item_2 = SubmenuItem("submenu_item_2", submenu2, menu=root_menu)

        root_menu.append_item(submenu_item_1)
        root_menu.append_item(submenu_item_2)

        root_menu.start()
        root_menu.wait_for_start(timeout=10)
        self.assertIs(ConsoleMenu.currently_active_menu, root_menu)
        submenu_item_1.action()
        submenu1.wait_for_start(timeout=10)
        self.assertIs(ConsoleMenu.currently_active_menu, submenu1)
        ConsoleMenu.currently_active_menu.exit()
        submenu1.join(timeout=10)
        self.assertIs(ConsoleMenu.currently_active_menu, root_menu)
        submenu_item_2.action()
        submenu2.wait_for_start(timeout=10)
        self.assertIs(ConsoleMenu.currently_active_menu, submenu2)



from base_test_case import BaseTestCase
from consolemenu import ConsoleMenu
from consolemenu.items import MenuItem


class TestSampleMenu(BaseTestCase):

    def setUp(self):
        super(TestSampleMenu, self).setUp()

        self.menu = ConsoleMenu("self.menu", "TestSampleMenu")
        self.item1 = MenuItem("self.item1", self.menu)
        self.item2 = MenuItem("self.item2", self.menu)
        self.menu.append_item(self.item1)
        self.menu.append_item(self.item2)
        self.menu.start()
        self.menu.wait_for_start(timeout=10)

    def tearDown(self):
        super(TestSampleMenu, self).tearDown()
        self.menu.exit()
        self.menu.join(timeout=10)

    def test_go_down(self):
        self.menu.go_down()
        self.assertEqual(self.menu.current_option, 1)
        self.assertIs(self.menu.current_item, self.item2)
        self.menu.go_down()
        self.assertEqual(self.menu.current_option, 2)
        self.assertEqual(self.menu.current_item, self.menu.exit_item)
        self.menu.go_down()
        self.assertEqual(self.menu.current_option, 0)
        self.assertIs(self.menu.current_item, self.item1)

    def test_go_up(self):
        self.menu.go_up()
        self.assertEqual(self.menu.current_option, 2)
        self.assertIs(self.menu.current_item, self.menu.exit_item)
        self.menu.go_up()
        self.assertEqual(self.menu.current_option, 1)
        self.assertEqual(self.menu.current_item, self.item2)
        self.menu.go_up()
        self.assertEqual(self.menu.current_option, 0)
        self.assertIs(self.menu.current_item, self.item1)

    def test_go_to(self):
        self.menu.go_to(1)
        self.assertEqual(self.menu.current_option, 1)
        self.assertEqual(self.menu.current_item, self.item2)

    def test_select(self):
        self.menu.select()
        self.assertEqual(self.menu.selected_option, 0)
        self.assertIs(self.menu.selected_item, self.item1)
        self.menu.go_down()
        self.menu.select()
        self.assertEqual(self.menu.selected_option, 1)
        self.assertIs(self.menu.selected_item, self.item2)
        self.menu.go_down()
        self.menu.select()
        self.assertEqual(self.menu.selected_option, 2)
        self.assertIs(self.menu.selected_item, self.menu.exit_item)
        self.menu.join(timeout=10)
        self.assertFalse(self.menu.is_alive())

    def test_exit(self):
        self.assertTrue(self.menu.is_alive())
        self.menu.exit()
        self.menu.join(timeout=10)
        self.assertFalse(self.menu.is_alive())


class TestConsoleMenu(BaseTestCase):

    def setUp(self):
        super(TestConsoleMenu, self).setUp()
        ConsoleMenu.currently_active_menu = None

    def test_init(self):
        menu1 = ConsoleMenu()
        menu2 = ConsoleMenu("menu2", "test_init", True)
        menu3 = ConsoleMenu(title="menu3", subtitle="test_init", show_exit_option=False)
        self.assertIsNone(menu1.title)
        self.assertEqual(menu2.title, "menu2")
        self.assertEqual(menu3.title, "menu3")
        self.assertIsNone(menu1.subtitle)
        self.assertEqual(menu2.subtitle, "test_init")
        self.assertEqual(menu3.subtitle, "test_init")
        self.assertTrue(menu1.show_exit_option)
        self.assertTrue(menu2.show_exit_option)
        self.assertFalse(menu3.show_exit_option)

    def test_currently_active_menu(self):
        menu1 = ConsoleMenu("menu1", "test_currently_active_menu_1")
        menu2 = ConsoleMenu("menu2", "test_currently_active_menu_2")
        menu3 = ConsoleMenu("menu3", "test_currently_active_menu_3")
        self.assertIsNone(ConsoleMenu.currently_active_menu)
        menu1.start()
        menu1.wait_for_start(10)
        self.assertIs(ConsoleMenu.currently_active_menu, menu1)
        menu2.start()
        menu2.wait_for_start(10)
        self.assertIs(ConsoleMenu.currently_active_menu, menu2)
        menu3.start()
        menu3.wait_for_start(10)
        self.assertIs(ConsoleMenu.currently_active_menu, menu3)
        menu3.join(timeout=5)
        menu2.join(timeout=5)
        menu1.join(timeout=5)

    def test_remove_menu_item(self):
        menu = ConsoleMenu("menu1", "test_currently_active_menu_1")
        item1 = MenuItem(text='itemText', menu=menu)
        item2 = MenuItem(text='itemText2', menu=menu)
        menu.append_item(item1)
        menu.append_item(item2)
        self.assertIn(item1, menu.items)
        self.assertIn(item2, menu.items)
        menu.remove_item(item1)
        self.assertNotIn(item1, menu.items)
        self.assertIn(item2, menu.items)



from base_test_case import BaseTestCase
from consolemenu import ConsoleMenu
from consolemenu.items import MenuItem, SubmenuItem

title_text = "title"
subtitle_text = "subtitle"
item_title_text = "itemtext"
submenu_title_text = "mysub"
count = 0


def menu_title():
    return title_text


def menu_subtitle():
    return subtitle_text


def item_title():
    return item_title_text


def submenu_title():
    return submenu_title_text


def my_submenu_function():
    global count
    if count == 0:
        count = count + 1
        return ConsoleMenu("version1")
    else:
        return ConsoleMenu("version2")


class TestDynamicTexts(BaseTestCase):
    def setUp(self):
        super(TestDynamicTexts, self).setUp()
        self.menu = ConsoleMenu(menu_title, menu_subtitle)
        self.first_call = True

    def test_(self):
        global title_text, subtitle_text, item_title_text
        menu_item = MenuItem(item_title, self.menu)

        # Test dynamic change of main menu title
        self.assertEqual(self.menu.get_title(), "title")
        title_text = "newtitle"
        self.assertEqual(self.menu.get_title(), "newtitle")

        # Test dynamic change of main menu subtitle
        self.assertEqual(self.menu.get_subtitle(), "subtitle")
        subtitle_text = "newsubtitle"
        self.assertEqual(self.menu.get_subtitle(), "newsubtitle")

        # Test dynamic menu item text
        self.assertEqual(menu_item.get_text(), "itemtext")
        item_title_text = "newtext"
        self.assertEqual(menu_item.get_text(), "newtext")

        # Test dynamic change of submenu item on each invocation
        submenu_item = SubmenuItem("a_sub", my_submenu_function)
        self.assertEqual(submenu_item.get_submenu().get_title(), "version1")
        self.assertEqual(submenu_item.get_submenu().get_title(), "version2")



import io

import re
from setuptools import setup, find_packages


def read(*filenames, **kwargs):
    encoding = kwargs.get('encoding', 'utf-8')
    sep = kwargs.get('sep', '\n')
    buf = []
    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())
    return sep.join(buf)


version_file_name = "consolemenu/version.py"
version_file_contents = open("consolemenu/version.py", "rt").read()
version_regex = r"^__version__ = ['\"]([^'\"]*)['\"]"
match = re.search(version_regex, version_file_contents, re.M)
if match:
    __version__ = match.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % (version_file_name,))

setup(
    name='console-menu',
    version=__version__,
    url='http://github.com/aegirhall/console-menu',
    license='MIT',
    author='Aegir Hall',
    author_email='aegirhall@gmail.com',
    description='A simple console menu system',
    long_description=read("README.rst", "CHANGELOG.rst"),
    packages=find_packages(),
    install_requires=['six', 'ansiwrap'],
    # setup_requires=['pytest-runner'],
    # tests_require=['tox'],
    # cmdclass={'test': Tox},
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console :: Curses',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



from unittest import TestCase

from consolemenu.validators.base import BaseValidator


class BaseTestCase(TestCase):
    def test_is_abstract(self):
        with self.assertRaises(TypeError):
            BaseValidator()

    def test_validate_is_present(self):
        self.assertIsNotNone(getattr(BaseValidator, 'validate'))



from __future__ import print_function

from base_test_case import BaseTestCase
from consolemenu import ConsoleMenu
from consolemenu.format.menu_style import (
    MenuStyle
)
from consolemenu.format.menu_borders import *
from consolemenu.format.menu_padding import MenuPadding
from consolemenu.format.menu_margins import MenuMargins
from consolemenu.items import MenuItem
from consolemenu.menu_component import MenuHeader, MenuTextSection, MenuItemsSection, MenuFooter, MenuPrompt


def print_screen_edge(width=80):
    msg = '{title:{fill}^{width}}'.format(title='simulate screen edges', fill='-', width=(width - 2))
    print('{edge}{msg}{edge}'.format(edge="|", msg=msg))


class TestMenuHeader(BaseTestCase):

    def test_menu_header_ascii_no_titles(self):
        print_screen_edge()
        header = MenuHeader(MenuStyle(border_style_type=MenuBorderStyleType.ASCII_BORDER))
        for line in header.generate():
            print(line)

    def test_menu_header_ascii_no_subtitle(self):
        print_screen_edge()
        header = MenuHeader(MenuStyle(border_style_type=MenuBorderStyleType.ASCII_BORDER), title="My Title")
        for line in header.generate():
            print(line)

    def test_menu_header_ascii(self):
        print_screen_edge()
        header = MenuHeader(MenuStyle(border_style_type=MenuBorderStyleType.ASCII_BORDER),
                            title="My Title", subtitle="My Subtitle")
        for line in header.generate():
            print(line)

    def test_menu_header_ascii_title_centered(self):
        print_screen_edge()
        header = MenuHeader(MenuStyle(border_style_type=MenuBorderStyleType.ASCII_BORDER),
                            title="My Centered Title", title_align='center')
        for line in header.generate():
            print(line)

    def test_menu_header_ascii_titles_centered(self):
        print_screen_edge()
        header = MenuHeader(MenuStyle(border_style_type=MenuBorderStyleType.ASCII_BORDER),
                            title="My Centered Title", title_align='center',
                            subtitle="My Centered Subtitle", subtitle_align='center')
        for line in header.generate():
            print(line)

    def test_menu_header_ascii_with_bottom_border(self):
        print_screen_edge()
        header = MenuHeader(MenuStyle(border_style_type=MenuBorderStyleType.ASCII_BORDER),
                            title="My Title", subtitle="I Should Have a Bottom Border",
                            show_bottom_border=True)
        for line in header.generate():
            print(line)

    def test_menu_header_light_no_titles(self):
        print_screen_edge()
        header = MenuHeader(MenuStyle(border_style=LightBorderStyle()))
        for line in header.generate():
            print(line)

    def test_menu_header_light_no_subtitle(self):
        print_screen_edge()
        header = MenuHeader(MenuStyle(border_style=LightBorderStyle()), title="My Title")
        for line in header.generate():
            print(line)

    def test_menu_header_light_margins_t1_l4_b0_r4(self):
        print_screen_edge()
        header = MenuHeader(MenuStyle(margins=MenuMargins(1, 4, 0, 4),
                                      border_style=LightBorderStyle()),
                            title="My Title")
        for line in header.generate():
            print(line)

    def test_menu_header_light_padding_t3_l4_b3_r4(self):
        print_screen_edge()
        header = MenuHeader(MenuStyle(padding=MenuPadding(3, 4, 3, 4),
                                      border_style=LightBorderStyle()),
                            title="My Title")
        for line in header.generate():
            print(line)

    def test_menu_header_light_with_bottom_border(self):
        print_screen_edge()
        header = MenuHeader(MenuStyle(border_style_type=MenuBorderStyleType.LIGHT_BORDER),
                            title="My Title", subtitle="I Should Have a Bottom Border",
                            show_bottom_border=True)
        for line in header.generate():
            print(line)


class TestMenuTextBlock(BaseTestCase):

    def setUp(self):
        self.long_text = "This is my really long text. I can use this text area to place very long \
paragraphs of text, to describe to the user what the menu is for, or even to describe each individual menu option, \
if I think the user is not smart enough to figure all this out on his or her own. I hope this is paragraph \
is long enough to display my multi-line capabilities."

    def test_menu_prologue_ascii(self):
        print_screen_edge()
        pro = MenuTextSection(MenuStyle())
        for line in pro.generate():
            print(line)

    def test_menu_prologue_ascii_no_text_with_top_border(self):
        print_screen_edge()
        pro = MenuTextSection(MenuStyle(), show_top_border=True)
        for line in pro.generate():
            print(line)

    def test_menu_prologue_ascii_no_text_with_bottom_border(self):
        print_screen_edge()
        pro = MenuTextSection(MenuStyle(), show_bottom_border=True)
        for line in pro.generate():
            print(line)

    def test_menu_prologue_ascii_no_text_with_both_borders(self):
        print_screen_edge()
        pro = MenuTextSection(MenuStyle(), show_top_border=True, show_bottom_border=True)
        for line in pro.generate():
            print(line)

    def test_menu_prologue_ascii_text_with_top_border(self):
        print_screen_edge()
        pro = MenuTextSection(MenuStyle(), text="This is my really good prologue.", show_top_border=True)
        for line in pro.generate():
            print(line)

    def test_menu_prologue_ascii_text_with_bottom_border(self):
        print_screen_edge()
        pro = MenuTextSection(MenuStyle(), text="This is my really good prologue.", show_bottom_border=True)
        for line in pro.generate():
            print(line)

    def test_menu_prologue_unicodelight(self):
        print_screen_edge()
        pro = MenuTextSection(MenuStyle(border_style=LightBorderStyle()))
        for line in pro.generate():
            print(line)

    def test_menu_prologue_unicodelight_with_top_border(self):
        print_screen_edge()
        pro = MenuTextSection(MenuStyle(border_style=LightBorderStyle()), show_top_border=True)
        for line in pro.generate():
            print(line)

    def test_menu_prologue_unicodelight_centered(self):
        print_screen_edge()
        pro = MenuTextSection(MenuStyle(border_style=LightBorderStyle()),
                              text="My centered prologue.", text_align="center")
        for line in pro.generate():
            print(line)

    def test_menu_prologue_unicodelight_long_text_default_margins_and_padding(self):
        print_screen_edge()
        pro = MenuTextSection(MenuStyle(border_style=LightBorderStyle()),
                              text=self.long_text)
        for line in pro.generate():
            print(line)

    def test_menu_prologue_unicodelight_long_text_big_margins(self):
        print_screen_edge()
        pro = MenuTextSection(MenuStyle(margins=MenuMargins(1, 8, 1, 8), border_style=LightBorderStyle()),
                              text=self.long_text)
        for line in pro.generate():
            print(line)

    def test_menu_prologue_unicodelight_long_text_big_padding(self):
        print_screen_edge()
        pro = MenuTextSection(MenuStyle(padding=MenuPadding(2, 8, 2, 8), border_style=LightBorderStyle()),
                              text=self.long_text)
        for line in pro.generate():
            print(line)

    def test_menu_prologue_unicodelight_long_text_big_margins_and_big_padding(self):
        print_screen_edge()
        pro = MenuTextSection(MenuStyle(margins=MenuMargins(1, 8, 1, 8), padding=MenuPadding(2, 8, 2, 8),
                                        border_style=LightBorderStyle()),
                              text=self.long_text)
        for line in pro.generate():
            print(line)

    def test_menu_prologue_unicodelight_long_text_set_via_property(self):
        print_screen_edge()
        pro = MenuTextSection(MenuStyle(border_style=LightBorderStyle()))
        pro.text = self.long_text
        for line in pro.generate():
            print(line)


class TestMenuItemsSection(BaseTestCase):

    def setUp(self):
        self.menu = ConsoleMenu("self.menu", "TestMenuItem")
        self.small_list = [
            MenuItem("menu_item_1", self.menu),
            MenuItem("menu_item_2", self.menu, True),
            MenuItem(text="menu_item_3", menu=self.menu, should_exit=False)
        ]
        self.large_list = [
            MenuItem("menu_item_1", self.menu),
            MenuItem("menu_item_2", self.menu, True),
            MenuItem("menu_item_3", self.menu, True),
            MenuItem("menu_item_4", self.menu, True),
            MenuItem("menu_item_5", self.menu, True),
            MenuItem("menu_item_6", self.menu, True),
            MenuItem("menu_item_7", self.menu, True),
            MenuItem("menu_item_8", self.menu, True),
            MenuItem("menu_item_9", self.menu, True),
            MenuItem("menu_item_10", self.menu, True),
            MenuItem("menu_item_11", self.menu, True),
            MenuItem(text="menu_item_12", menu=self.menu, should_exit=False)
        ]

    def test_menu_items_ascii_no_items(self):
        print_screen_edge()
        sect = MenuItemsSection(MenuStyle())
        for line in sect.generate():
            print(line)

    def test_menu_items_ascii(self):
        print_screen_edge()
        sect = MenuItemsSection(MenuStyle(), items=self.small_list)
        for line in sect.generate():
            print(line)

    def test_menu_items_ascii_large_list(self):
        print_screen_edge()
        sect = MenuItemsSection(MenuStyle(), items=self.large_list)
        for line in sect.generate():
            print(line)

    def test_menu_items_ascii_padding(self):
        print_screen_edge()
        sect = MenuItemsSection(MenuStyle(padding=MenuPadding(1, 4, 1, 4)), items=self.large_list)
        for line in sect.generate():
            print(line)

    def test_menu_items_ascii_margins_and_padding(self):
        print_screen_edge()
        sect = MenuItemsSection(MenuStyle(margins=MenuMargins(2, 8, 2, 8), padding=MenuPadding(1, 8, 1, 8)),
                                items=self.small_list)
        for line in sect.generate():
            print(line)

    def test_menu_items_ascii_padding_large_list(self):
        print_screen_edge()
        sect = MenuItemsSection(MenuStyle(padding=MenuPadding(1, 4, 1, 4)), items=self.large_list)
        for line in sect.generate():
            print(line)

    def test_menu_items_unicodelight(self):
        print_screen_edge()
        sect = MenuItemsSection(MenuStyle(border_style=LightBorderStyle()), items=self.small_list)
        for line in sect.generate():
            print(line)

    def test_show_bottom_border_for_items(self):
        print_screen_edge()
        sect = MenuItemsSection(MenuStyle(border_style=LightBorderStyle()), items=self.large_list)
        sect.show_item_bottom_border('menu_item_4', True)
        sect.show_item_bottom_border('menu_item_8', True)
        sect.show_item_bottom_border('menu_item_12', True)
        for line in sect.generate():
            print(line)

    def test_show_top_border_for_items(self):
        print_screen_edge()
        sect = MenuItemsSection(MenuStyle(border_style=LightBorderStyle()), items=self.large_list)
        sect.show_item_top_border('menu_item_4', True)
        sect.show_item_top_border('menu_item_8', True)
        sect.show_item_top_border('menu_item_12', True)
        for line in sect.generate():
            print(line)

    def test_show_top_and_bottom_borders_for_items(self):
        print_screen_edge()
        sect = MenuItemsSection(MenuStyle(border_style=LightBorderStyle()), items=self.large_list)
        sect.show_item_top_border('menu_item_4', True)
        sect.show_item_bottom_border('menu_item_4', True)
        sect.show_item_top_border('menu_item_8', True)
        sect.show_item_bottom_border('menu_item_8', True)
        sect.show_item_top_border('menu_item_12', True)
        sect.show_item_bottom_border('menu_item_12', True)
        for line in sect.generate():
            print(line)

    def test_show_both_borders_for_items_then_disable_borders(self):
        print_screen_edge()
        sect = MenuItemsSection(MenuStyle(border_style=LightBorderStyle()), items=self.large_list)
        sect.show_item_top_border('menu_item_4', True)
        sect.show_item_bottom_border('menu_item_4', True)
        sect.show_item_top_border('menu_item_8', True)
        sect.show_item_bottom_border('menu_item_8', True)
        sect.show_item_top_border('menu_item_12', True)
        sect.show_item_bottom_border('menu_item_12', True)
        print("This should show top and bottom borders on items 4, 8, and 12")
        for line in sect.generate():
            print(line)
        sect.show_item_top_border('menu_item_4', False)
        sect.show_item_bottom_border('menu_item_4', False)
        sect.show_item_top_border('menu_item_8', False)
        sect.show_item_bottom_border('menu_item_8', False)
        sect.show_item_top_border('menu_item_12', False)
        sect.show_item_bottom_border('menu_item_12', False)
        print("This should not show any borders on any item")
        for line in sect.generate():
            print(line)


class TestMenuFooter(BaseTestCase):

    def test_menu_footer_ascii(self):
        footer = MenuFooter(MenuStyle())
        for line in footer.generate():
            print(line)
        print_screen_edge()

    def test_menu_footer_ascii_margins_t8_l8_b4_r8(self):
        footer = MenuFooter(MenuStyle(margins=MenuMargins(8, 8, 4, 8)))
        for line in footer.generate():
            print(line)
        print_screen_edge()

    def test_menu_footer_unicodelight(self):
        footer = MenuFooter(MenuStyle())
        for line in footer.generate():
            print(line)
        print_screen_edge()


class TestMenuPrompt(BaseTestCase):

    def test_ascii_prompt(self):
        footer = MenuFooter(MenuStyle())
        mp = MenuPrompt(MenuStyle())
        for line in footer.generate():
            print(line)
        for line in mp.generate():
            print(line)

    def test_ascii_prompt_no_top_margin(self):
        # Set footer's bottom margin to 0 and prompt's top padding to 0 so prompt is touching bottom of menu.
        footer = MenuFooter(MenuStyle(margins=MenuMargins(0, 2, 0, 2)))
        mp = MenuPrompt(MenuStyle(padding=MenuPadding(0, 1, 1, 1)))
        for line in footer.generate():
            print(line)
        for line in mp.generate():
            print(line)

    def test_unicode_prompt_no_top_margin(self):
        # Set top padding to 0 so prompt is touching bottom of menu.
        style = MenuStyle(padding=MenuPadding(0, 2, 0, 2), border_style=LightBorderStyle())
        footer = MenuFooter(style)
        mp = MenuPrompt(style, prompt_string=u"\u27EB")
        for line in footer.generate():
            print(line)
        for line in mp.generate():
            print(line)



from base_test_case import BaseTestCase
from consolemenu.format import *


class DummyMenuBorderStyle(LightBorderStyle):
    """ Dummy class to use for testing border style overrides in MenuStyle. """


class DummyMenuBorderStyleFactory(MenuBorderStyleFactory):
    """ Dummy class to use for testing menu border overrides in MenuStyle. """
    def create_border(self, border_style_type):
        if border_style_type == 8:
            return DummyMenuBorderStyle()
        else:
            return super(DummyMenuBorderStyleFactory, self).create_border(border_style_type)


class TestMenuStyle(BaseTestCase):

    def test_defaults(self):
        ms = MenuStyle()
        # Default Border Factory should be MenuBorderStyleFactory
        self.assertTrue(isinstance(ms.border_style_factory, MenuBorderStyleFactory))
        # Default Border style should be LightBorderStyle
        self.assertTrue(isinstance(ms.border_style, LightBorderStyle))
        # Default Margins
        self.assertTrue(isinstance(ms.margins, MenuMargins))
        self.assertEqual(1, ms.margins.top)
        self.assertEqual(2, ms.margins.left)
        self.assertEqual(0, ms.margins.bottom)
        self.assertEqual(2, ms.margins.right)
        # Default Padding
        self.assertTrue(isinstance(ms.padding, MenuPadding))
        self.assertEqual(1, ms.padding.top)
        self.assertEqual(2, ms.padding.left)
        self.assertEqual(1, ms.padding.bottom)
        self.assertEqual(2, ms.padding.right)

    def test_constructor_margins_and_padding(self):
        ms = MenuStyle(margins=MenuMargins(8, 7, 6, 5), padding=MenuPadding(12, 11, 10, 9))
        # Default Border Factory should be MenuBorderStyleFactory
        self.assertTrue(isinstance(ms.border_style_factory, MenuBorderStyleFactory))
        # Default Border style should be LightBorderStyle
        self.assertTrue(isinstance(ms.border_style, LightBorderStyle))
        # Given Margins
        self.assertTrue(isinstance(ms.margins, MenuMargins))
        self.assertEqual(8, ms.margins.top)
        self.assertEqual(7, ms.margins.left)
        self.assertEqual(6, ms.margins.bottom)
        self.assertEqual(5, ms.margins.right)
        # Given Padding
        self.assertTrue(isinstance(ms.padding, MenuPadding))
        self.assertEqual(12, ms.padding.top)
        self.assertEqual(11, ms.padding.left)
        self.assertEqual(10, ms.padding.bottom)
        self.assertEqual(9, ms.padding.right)

    def test_constructor_border_style_only(self):
        ms = MenuStyle(border_style=HeavyBorderStyle())
        self.assertTrue(isinstance(ms.border_style, HeavyBorderStyle))
        self.assertTrue(isinstance(ms.border_style_factory, MenuBorderStyleFactory))

    def test_constructor_border_style_type_only(self):
        ms = MenuStyle(border_style_type=MenuBorderStyleType.DOUBLE_LINE_BORDER)
        self.assertTrue(isinstance(ms.border_style, DoubleLineBorderStyle))
        self.assertTrue(isinstance(ms.border_style_factory, MenuBorderStyleFactory))

    def test_constructor_border_factory_only(self):
        ms = MenuStyle(border_style_factory=DummyMenuBorderStyleFactory())
        self.assertTrue(isinstance(ms.border_style, LightBorderStyle))
        self.assertTrue(isinstance(ms.border_style_factory, DummyMenuBorderStyleFactory))

    def test_constructor_conflicting_border_class_and_type(self):
        ms = MenuStyle(border_style=HeavyBorderStyle(), border_style_type=MenuBorderStyleType.LIGHT_BORDER)
        # Class should take precedence
        self.assertTrue(isinstance(ms.border_style, HeavyBorderStyle))
        self.assertTrue(isinstance(ms.border_style_factory, MenuBorderStyleFactory))

    def test_constructor_conflicting_border_factory_and_type(self):
        ms = MenuStyle(border_style_factory=DummyMenuBorderStyleFactory(), border_style_type=8)
        self.assertTrue(isinstance(ms.border_style, DummyMenuBorderStyle))
        self.assertTrue(isinstance(ms.border_style_factory, DummyMenuBorderStyleFactory))



from consolemenu import ConsoleMenu

try:
    from unittest.mock import MagicMock
except ImportError:
    from mock import MagicMock

from consolemenu.items import FunctionItem
from test_external_item import TestExternalItem


def fun1():
    return 10


def fun2(x):
    return x + 2


class TestFunctionItem(TestExternalItem):
    def setUp(self):
        super(TestFunctionItem, self).setUp()
        self.menu = ConsoleMenu("self.menu", "TestFunctionItem")

    def test_run(self):
        mock_function_1 = MagicMock(name="mock_function_1", return_value=5)
        mock_function_2 = MagicMock(name="mock_function_2", return_value=10)

        args = [1, 2, 3]
        kwargs = {"end": "\n", "sep": " "}
        function_item_1 = FunctionItem("function_item_1", mock_function_1)
        function_item_2 = FunctionItem("function_item_2", mock_function_2, args, kwargs)
        function_item_1.action()
        function_item_2.action()

        self.assertEqual(function_item_1.get_return(), 5)
        self.assertEqual(function_item_2.get_return(), 10)
        mock_function_1.assert_any_call()
        mock_function_2.assert_called_once_with(*args, **kwargs)

    def test_init(self):
        function_item_1 = FunctionItem("function_item_1", fun1)
        function_item_2 = FunctionItem("function_item_2", fun1, ["-l", "-a", "~"], {"test": 12}, self.menu, True)
        function_item_3 = FunctionItem(text="function_item_3", function=fun2, args=[1, 2, 3],
                                       kwargs={1: "thing", 16: "other"}, menu=self.menu, should_exit=False)
        self.assertEqual(function_item_1.text, "function_item_1")
        self.assertEqual(function_item_2.text, "function_item_2")
        self.assertEqual(function_item_3.text, "function_item_3")
        self.assertIsNone(function_item_1.menu)
        self.assertEqual(function_item_2.menu, self.menu)
        self.assertEqual(function_item_3.menu, self.menu)
        self.assertFalse(function_item_1.should_exit)
        self.assertTrue(function_item_2.should_exit)
        self.assertFalse(function_item_3.should_exit)
        self.assertEqual(function_item_1.function, fun1)
        self.assertEqual(function_item_2.function, fun1)
        self.assertEqual(function_item_3.function, fun2)
        self.assertEqual(function_item_1.args, [])
        self.assertEqual(function_item_2.args, ["-l", "-a", "~"])
        self.assertEqual(function_item_3.args, [1, 2, 3])
        self.assertEqual(function_item_1.kwargs, {})
        self.assertEqual(function_item_2.kwargs, {"test": 12})
        self.assertEqual(function_item_3.kwargs, {1: "thing", 16: "other"})
